'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _Provider = require('./Provider');

var _Provider2 = _interopRequireDefault(_Provider);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('StripeProvider', function () {
  var stripeMockFn = void 0;
  var stripeMockResult = void 0;

  beforeEach(function () {
    stripeMockResult = {
      elements: jest.fn(),
      createToken: jest.fn(),
      createSource: jest.fn()
    };
    stripeMockFn = jest.fn().mockReturnValue(stripeMockResult);
    window.Stripe = stripeMockFn;
  });

  it('requires apiKey', function () {
    var originalConsoleError = global.console.error;
    var mockConsoleError = jest.fn();
    global.console.error = mockConsoleError;
    (0, _enzyme.shallow)(_react2.default.createElement(
      _Provider2.default,
      null,
      _react2.default.createElement('div', null)
    ));
    expect(mockConsoleError.mock.calls[0][0]).toContain('Warning: Failed prop type: The prop `apiKey` is marked as required in `Provider`, but its value is `undefined`.');
    global.console.error = originalConsoleError;
  });

  it('throws without stripe.js loaded', function () {
    window.Stripe = null;
    expect(function () {
      return (0, _enzyme.shallow)(_react2.default.createElement(_Provider2.default, { apiKey: 'made_up_key' }));
    }).toThrow('Please load Stripe.js (https://js.stripe.com/v3/) on this page to use react-stripe-elements.');
  });

  it('throws without children', function () {
    expect(function () {
      return (0, _enzyme.shallow)(_react2.default.createElement(_Provider2.default, { apiKey: 'made_up_key' }));
    }).toThrow('React.Children.only expected to receive a single React element child');
  });

  it('throws with more than one children', function () {
    expect(function () {
      return (0, _enzyme.shallow)(_react2.default.createElement(
        _Provider2.default,
        { apiKey: 'made_up_key' },
        _react2.default.createElement('div', null),
        _react2.default.createElement('div', null)
      ));
    }).toThrow('React.Children.only expected to receive a single React element child');
  });

  it('renders its single child', function () {
    var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'made_up_key' },
      _react2.default.createElement(
        'form',
        null,
        _react2.default.createElement('input', null)
      )
    ));

    expect(wrapper.html()).toBe('<form><input/></form>');
  });

  it('initializes Stripe with apiKey and empty options', function () {
    (0, _enzyme.shallow)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'made_up_key' },
      _react2.default.createElement('form', null)
    ));
    expect(stripeMockFn).toHaveBeenCalledWith('made_up_key', {});
  });

  it('initializes Stripe with apiKey and arbitrary props as options', function () {
    (0, _enzyme.shallow)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'made_up_key', foo: 'bar' },
      _react2.default.createElement('form', null)
    ));
    expect(stripeMockFn).toHaveBeenCalledWith('made_up_key', { foo: 'bar' });
  });

  it('provides context.stripe', function () {
    var wrapper = (0, _enzyme.mount)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'made_up_key' },
      _react2.default.createElement('form', null)
    ));
    var childContext = wrapper.node.getChildContext();
    expect(childContext).toEqual({ stripe: stripeMockResult });
  });

  it('does not create a new Stripe instance if one exists for the same key', function () {
    window.Stripe = jest.fn(function () {
      return {};
    });

    // First, create the first instance.
    var wrapper = (0, _enzyme.mount)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'key_one' },
      _react2.default.createElement('form', null)
    ));
    var childContext = wrapper.node.getChildContext();
    var keyOneInstance = childContext.stripe;
    expect(keyOneInstance).toBeTruthy();

    // Create another!
    wrapper = (0, _enzyme.mount)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'key_one' },
      _react2.default.createElement('form', null)
    ));
    childContext = wrapper.node.getChildContext();
    expect(childContext.stripe).toBe(keyOneInstance);

    // Create another, but with a different key!
    wrapper = (0, _enzyme.mount)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'key_two' },
      _react2.default.createElement('form', null)
    ));
    childContext = wrapper.node.getChildContext();
    expect(childContext.stripe).not.toBe(keyOneInstance);
  });

  it('warns when trying to change the API key', function () {
    var originalConsoleError = global.console.error;
    var mockConsoleError = jest.fn();
    global.console.error = mockConsoleError;
    var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(
      _Provider2.default,
      { apiKey: 'made_up_key' },
      _react2.default.createElement('form', null)
    ));
    wrapper.setProps({ apiKey: 'a_new_key' });
    expect(mockConsoleError).toHaveBeenCalledWith('StripeProvider does not support changing the apiKey parameter.');
    global.console.error = originalConsoleError;
  });
}); // @noflow